

-- UUAGC 0.9.8 (Ag.ag)
module CCO.Imp.AG where

{-# LINE 2 "./AG/CodeGeneration.ag" #-}

import CCO.Ssm hiding (Add, Sub, Mul, Div, Eq, Lt, Gt)
import Prelude hiding (div)
{-# LINE 11 "Ag.hs" #-}
{-# LINE 5 "./AG/Base.ag" #-}

type Ident = String    -- ^ Type of identifiers.
{-# LINE 15 "Ag.hs" #-}

{-# LINE 47 "./AG/CodeGeneration.ag" #-}

-- | An environment maps identifiers to symbol descriptors.
type Env = [(Ident, Sym)]

-- | A symbol descriptor either describes a variable or a function symbol.
-- For a variable, we store its offset relative to the mark pointer; for
-- a function we store its begin label.
data Sym = V Int | F Int

-- | Restrict environments to a particular type of descriptor.
vars   env = [entry | entry@(_, V _     ) <- env            ]
funs   env = [entry | entry@(_, F _     ) <- env            ]
params env = [entry | entry@(_, V offset) <- env, offset < 0]
{-# LINE 31 "Ag.hs" #-}

{-# LINE 82 "./AG/CodeGeneration.ag" #-}

-- | A symbol table contains descriptors for each variable that is in scope at
-- a certain program point.
-- The table consists of levels of 'Env's reflecting the nesting of functions:
-- * The head of the list of 'Env's corresponds to the innermost function scope
--   and so it contains the descriptors of the symbols that are allocated
--   relative to the current mark pointer.
-- * To access the symbols described in the tail of the list, static links are
--   to be followed.
type Syms = [Env]
{-# LINE 44 "Ag.hs" #-}

{-# LINE 108 "./AG/CodeGeneration.ag" #-}

-- list for keep track of the used variables (identifier, occurrences)
type UsedVars = [(Ident, Int)]
{-# LINE 50 "Ag.hs" #-}

{-# LINE 194 "./AG/CodeGeneration.ag" #-}

-- | Merges two lists of used variables into one, avoiding duplicates and
-- | adjusting the usage counters of all variables.
mergeUsedVars :: UsedVars -> UsedVars -> UsedVars
mergeUsedVars []          ys = ys
mergeUsedVars ((x, n):xs) ys = mergeUsedVars xs (incrUsage x ys n)

-- | Looks up the Ident in given usage list and adds/increments the counter for
-- | that counter with the given number.
incrUsage :: Ident -> [(Ident, Int)] -> Int -> [(Ident, Int)]
incrUsage x []          incr = [(x, incr)]
incrUsage x ((y, n):ys) incr | y == x     = (y, (n + incr)) : ys
                             | otherwise  = (y, n) : incrUsage x ys incr

-- | Produces code for entering a block.
enter :: Env -> CodeS
enter env = foldr (.) id [ldc 0 | (_, V _) <- env]

-- | Cache static links for surrounding lexical levels of which variables are
-- | used more than once, except the closest level
cache :: Syms -> UsedVars -> CodeS
cache syms vars = if length syms > 1
                     then ldl (- 2) . lda (- 2)
                        . getLexAddress (tail syms) vars ([]++)
                     else ([]++) -- only one level which is the closest level;
                                 -- do nothing
    where -- Input: symbol table, used variables, code needed to prepend if
          -- previous level(s) were not used
          getLexAddress :: Syms -> UsedVars -> CodeS -> CodeS
          getLexAddress []     _    _   = ([]++) -- do nothing
          getLexAddress (s:ss) vars pre =
              let (used, vars') = lexLevelUsed vars s 0
              in if used
                    then pre . getLexAddress ss vars' (lds 0 . lda (-2))
                    else getLexAddress ss vars' (pre . lda (- 2))

-- | Determines if a given environment is being used from the context of the
-- | current used variables. It returns whether or not the environment is used
-- | and the list of variables that were not used in this environment. The last
-- | argument stated how many times this environment was already used besides
-- | the possible usages in UsedVars (used for calling this function
-- | recursively, when calling from the outside you should set this to 0.
lexLevelUsed :: UsedVars -> Env -> Int -> (Bool, UsedVars)
lexLevelUsed [] env c  = ((c>1), [])
lexLevelUsed ((x, n):xs) env c | find x env = lexLevelUsed xs env (c+n)
                               | otherwise  = let (found, idents) =
                                                      lexLevelUsed xs env c
                                              in (found, ((x, n):idents))
    where find :: Ident -> Env -> Bool
          find x env = case lookup x env of
                            Just _  -> True
                            Nothing -> False

-- | Produces code for exiting a block.
exit :: Env -> CodeS
exit env = ajs (- length (vars env))

-- | Cleanup static link cache
clean :: Syms -> UsedVars -> CodeS
clean syms uvars = ajs (min (- usedLevels syms uvars + 1) (-1))
    where usedLevels :: Syms -> UsedVars -> Int
          usedLevels [] _ = 0
          usedLevels (env:envs) usvars =
              let (used, usvars') = lexLevelUsed usvars env 0
              in if used
                    then 1 + usedLevels envs usvars'
                    else usedLevels envs usvars

-- | Produces code for retrieving the value of a variable.
get :: Ident -> Syms -> UsedVars -> CodeS
get x (local : global) uvars = case lookup x (vars local) of
        Nothing         -> getClosest x global skip uvars
        Just (V offset) -> ldl offset
       -- number of positions to skip to the static links
    where skip = (length $ vars local) - (length $ params local)

-- | Produces code for retrieving the value of a variable in the closest
-- | surrounding lexical level. Initializes the getGlobal function when
-- | neccessary with the first higher lexical level.
getClosest :: Ident -> Syms -> Int -> UsedVars -> CodeS
getClosest x (closest : global) s uvars = case lookup x (vars closest) of
        Nothing         -> getGlobal x global s (ldl (- 2) . lda (- 2)) uvars
        Just (V offset) -> ldl (- 2) . lda offset

-- | Produces code for retrieving the value of a global variable.
-- | Its arguments are the variable name, symbol table, nr of the
-- | cached environment (possibly some have not been cached), code
-- | for accessing the variable if the endpoint was not saved.
getGlobal :: Ident -> Syms -> Int -> CodeS -> UsedVars -> CodeS
getGlobal x []                 _ _ _ = error ("unknown variable: " ++ x)
getGlobal x (env : envs) s pre uvars = case lookup x (vars env) of
                  -- not found; continue with higher environments. in pre we
                  -- build a traversal for the case we find out that this
                  -- variable's endpoint was not cached
  Nothing         -> let (cached, uvars') = lexLevelUsed uvars env 0
                     in if cached
                           then getGlobal x envs (s + 1) (pre . lda (- 2)) uvars'
                           else getGlobal x envs s (pre . lda (- 2)) uvars'
                  -- found; skip local variables while loading static link, or
                  -- execute the traversal if this variable's endpoint was no
                  -- cached
  Just (V offset) -> let (cached, uvars') = lexLevelUsed uvars env 0
                     in if cached
                           then ldl (s + 1) . lda offset
                           else pre . lda offset

-- | Produces code for assigning a new value to a variable. For further comments
-- | look at get.
set :: Ident -> Syms -> UsedVars -> CodeS
set x (local : global) uvars = case lookup x (vars local) of
        Nothing         -> setClosest x global skip uvars
        Just (V offset) -> stl offset
       -- number of positions to skip to the static links
    where skip = (length $ vars local) - (length $ params local)

-- | Produces code for assigning a new value to a variable in the closest
-- | surrounding lexical level. For further comments look at getClosest.
setClosest :: Ident -> Syms -> Int -> UsedVars -> CodeS
setClosest x (closest : global) s uvars = case lookup x (vars closest) of
        Nothing         -> setGlobal x global s (ldl (- 2) . lda (- 2)) uvars
        Just (V offset) -> ldl (- 2) . sta offset

-- | Produces code for assigning a new value to a global variable. For further
-- | comments look at getGlobal.
setGlobal :: Ident -> Syms -> Int -> CodeS -> UsedVars -> CodeS
setGlobal x []                 _ _ _ = error ("unknown variable: " ++ x)
setGlobal x (env : envs) s pre uvars = case lookup x (vars env) of
                  -- not found; continue with higher environments. in pre we
                  -- build a traversal for the case we find out that this
                  -- variable's endpoint was not cached
  Nothing         -> let (cached, uvars') = lexLevelUsed uvars env 0
                     in if cached
                           then setGlobal x envs (s + 1) (pre . lda (- 2)) uvars'
                           else setGlobal x envs s (pre . lda (- 2)) uvars'
                  -- found; skip local variables while loading static link, or
                  -- execute the traversal if this variable's endpoint was not
                  -- cached
  Just (V offset) -> let (cached, uvars') = lexLevelUsed uvars env 0
                     in if cached
                           then ldl (s + 1) . sta offset
                           else pre . sta offset

-- | Produces code for calling a function.
call :: Ident -> Syms -> CodeS
call f (local : global) = case lookup f (funs local) of
  Nothing             -> ldl (- 2) . callGlobal f global
  Just (F beginLabel) -> ldr MP . ldcL beginLabel . jsr

-- | Produces code for calling a global function.
callGlobal :: Ident -> Syms -> CodeS
callGlobal f []           = error ("unknown function: " ++ f)
callGlobal f (env : envs) = case lookup f (funs env) of
  Nothing             -> lda (- 2) . callGlobal f envs
  Just (F beginLabel) -> ldcL beginLabel . jsr

-- | Produces code for returning from a function.
return_ :: Syms -> CodeS
return_ (local : _) =
  sts (- (length (vars local) + 3)) .
  ldrr SP MP .
  str MP .
  sts (- length (params local)) .
  ajs (- (length (params local) - 1)) .
  ret
{-# LINE 217 "Ag.hs" #-}
-- Decl --------------------------------------------------------
data Decl  = FunDecl (Ident) ([Ident]) (Stmts) 
           | VarDecl (Ident) 
-- cata
sem_Decl :: Decl  ->
            T_Decl 
sem_Decl (FunDecl _f _xs _b )  =
    (sem_Decl_FunDecl _f _xs (sem_Stmts _b ) )
sem_Decl (VarDecl _x )  =
    (sem_Decl_VarDecl _x )
-- semantic domain
type T_Decl  = ([Label]) ->
               Int ->
               Syms ->
               UsedVars ->
               ( CodeS,Env,([Label]),Int,UsedVars)
data Inh_Decl  = Inh_Decl {labels_Inh_Decl :: [Label],offset_Inh_Decl :: Int,syms_Inh_Decl :: Syms,uvars_Inh_Decl :: UsedVars}
data Syn_Decl  = Syn_Decl {codes_Syn_Decl :: CodeS,env_Syn_Decl :: Env,labels_Syn_Decl :: [Label],offset_Syn_Decl :: Int,vars_Syn_Decl :: UsedVars}
wrap_Decl :: T_Decl  ->
             Inh_Decl  ->
             Syn_Decl 
wrap_Decl sem (Inh_Decl _lhsIlabels _lhsIoffset _lhsIsyms _lhsIuvars )  =
    (let ( _lhsOcodes,_lhsOenv,_lhsOlabels,_lhsOoffset,_lhsOvars) =
             (sem _lhsIlabels _lhsIoffset _lhsIsyms _lhsIuvars )
     in  (Syn_Decl _lhsOcodes _lhsOenv _lhsOlabels _lhsOoffset _lhsOvars ))
sem_Decl_FunDecl :: Ident ->
                    ([Ident]) ->
                    T_Stmts  ->
                    T_Decl 
sem_Decl_FunDecl f_ xs_ b_  =
    (\ _lhsIlabels
       _lhsIoffset
       _lhsIsyms
       _lhsIuvars ->
         (let _bOlabels :: ([Label])
              _bOoffset :: Int
              _lhsOenv :: Env
              _bOsyms :: Syms
              _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _bOuvars :: UsedVars
              _lhsOlabels :: ([Label])
              _lhsOoffset :: Int
              _bIcodes :: CodeS
              _bIenv :: Env
              _bIlabels :: ([Label])
              _bIoffset :: Int
              _bIvars :: UsedVars
              _beginLabel =
                  {-# LINE 19 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels !! 0
                  {-# LINE 269 "Ag.hs" #-}
              _endLabel =
                  {-# LINE 20 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels !! 1
                  {-# LINE 273 "Ag.hs" #-}
              _bOlabels =
                  {-# LINE 21 "./AG/CodeGeneration.ag" #-}
                  drop 2 _lhsIlabels
                  {-# LINE 277 "Ag.hs" #-}
              _bOoffset =
                  {-# LINE 40 "./AG/CodeGeneration.ag" #-}
                  1
                  {-# LINE 281 "Ag.hs" #-}
              _lhsOenv =
                  {-# LINE 69 "./AG/CodeGeneration.ag" #-}
                  [(f_, F _beginLabel    )]
                  {-# LINE 285 "Ag.hs" #-}
              _params =
                  {-# LINE 76 "./AG/CodeGeneration.ag" #-}
                  zipWith (\x i -> (x, V i)) xs_ [- (2 + length xs_) ..]
                  {-# LINE 289 "Ag.hs" #-}
              _bOsyms =
                  {-# LINE 100 "./AG/CodeGeneration.ag" #-}
                  (_params     ++ _bIenv) : _lhsIsyms
                  {-# LINE 293 "Ag.hs" #-}
              _lhsOcodes =
                  {-# LINE 128 "./AG/CodeGeneration.ag" #-}
                  bra _endLabel     .
                  label _beginLabel     .
                    ldr MP .
                    ldrr MP SP .
                    enter _bIenv .
                    cache _lhsIsyms _bIvars .
                    _bIcodes .
                    clean _lhsIsyms _bIvars .
                    exit _bIenv .
                    ldc 0 .
                    return_ (_params     : _lhsIsyms) .
                  label _endLabel
                  {-# LINE 308 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 142 "./AG/CodeGeneration.ag" #-}
                  []
                  {-# LINE 312 "Ag.hs" #-}
              _bOuvars =
                  {-# LINE 143 "./AG/CodeGeneration.ag" #-}
                  _bIvars
                  {-# LINE 316 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _bIlabels
                  {-# LINE 320 "Ag.hs" #-}
              _lhsOoffset =
                  {-# LINE 36 "./AG/CodeGeneration.ag" #-}
                  _bIoffset
                  {-# LINE 324 "Ag.hs" #-}
              ( _bIcodes,_bIenv,_bIlabels,_bIoffset,_bIvars) =
                  (b_ _bOlabels _bOoffset _bOsyms _bOuvars )
          in  ( _lhsOcodes,_lhsOenv,_lhsOlabels,_lhsOoffset,_lhsOvars)))
sem_Decl_VarDecl :: Ident ->
                    T_Decl 
sem_Decl_VarDecl x_  =
    (\ _lhsIlabels
       _lhsIoffset
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOoffset :: Int
              _lhsOenv :: Env
              _lhsOvars :: UsedVars
              _lhsOcodes :: CodeS
              _lhsOlabels :: ([Label])
              _lhsOoffset =
                  {-# LINE 39 "./AG/CodeGeneration.ag" #-}
                  _lhsIoffset + 1
                  {-# LINE 343 "Ag.hs" #-}
              _lhsOenv =
                  {-# LINE 68 "./AG/CodeGeneration.ag" #-}
                  [(x_, V _lhsIoffset    )]
                  {-# LINE 347 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 144 "./AG/CodeGeneration.ag" #-}
                  []
                  {-# LINE 351 "Ag.hs" #-}
              _lhsOcodes =
                  {-# LINE 114 "./AG/CodeGeneration.ag" #-}
                  id
                  {-# LINE 355 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 359 "Ag.hs" #-}
          in  ( _lhsOcodes,_lhsOenv,_lhsOlabels,_lhsOoffset,_lhsOvars)))
-- Decls -------------------------------------------------------
type Decls  = [(Decl)]
-- cata
sem_Decls :: Decls  ->
             T_Decls 
sem_Decls list  =
    (Prelude.foldr sem_Decls_Cons sem_Decls_Nil (Prelude.map sem_Decl list) )
-- semantic domain
type T_Decls  = ([Label]) ->
                Int ->
                Syms ->
                UsedVars ->
                ( CodeS,Env,([Label]),Int,UsedVars)
data Inh_Decls  = Inh_Decls {labels_Inh_Decls :: [Label],offset_Inh_Decls :: Int,syms_Inh_Decls :: Syms,uvars_Inh_Decls :: UsedVars}
data Syn_Decls  = Syn_Decls {codes_Syn_Decls :: CodeS,env_Syn_Decls :: Env,labels_Syn_Decls :: [Label],offset_Syn_Decls :: Int,vars_Syn_Decls :: UsedVars}
wrap_Decls :: T_Decls  ->
              Inh_Decls  ->
              Syn_Decls 
wrap_Decls sem (Inh_Decls _lhsIlabels _lhsIoffset _lhsIsyms _lhsIuvars )  =
    (let ( _lhsOcodes,_lhsOenv,_lhsOlabels,_lhsOoffset,_lhsOvars) =
             (sem _lhsIlabels _lhsIoffset _lhsIsyms _lhsIuvars )
     in  (Syn_Decls _lhsOcodes _lhsOenv _lhsOlabels _lhsOoffset _lhsOvars ))
sem_Decls_Cons :: T_Decl  ->
                  T_Decls  ->
                  T_Decls 
sem_Decls_Cons hd_ tl_  =
    (\ _lhsIlabels
       _lhsIoffset
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOenv :: Env
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _lhsOoffset :: Int
              _hdOlabels :: ([Label])
              _hdOoffset :: Int
              _hdOsyms :: Syms
              _hdOuvars :: UsedVars
              _tlOlabels :: ([Label])
              _tlOoffset :: Int
              _tlOsyms :: Syms
              _tlOuvars :: UsedVars
              _hdIcodes :: CodeS
              _hdIenv :: Env
              _hdIlabels :: ([Label])
              _hdIoffset :: Int
              _hdIvars :: UsedVars
              _tlIcodes :: CodeS
              _tlIenv :: Env
              _tlIlabels :: ([Label])
              _tlIoffset :: Int
              _tlIvars :: UsedVars
              _lhsOcodes =
                  {-# LINE 114 "./AG/CodeGeneration.ag" #-}
                  _hdIcodes . _tlIcodes
                  {-# LINE 417 "Ag.hs" #-}
              _lhsOenv =
                  {-# LINE 66 "./AG/CodeGeneration.ag" #-}
                  _hdIenv ++ _tlIenv
                  {-# LINE 421 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 116 "./AG/CodeGeneration.ag" #-}
                  _hdIvars `mergeUsedVars` _tlIvars
                  {-# LINE 425 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _tlIlabels
                  {-# LINE 429 "Ag.hs" #-}
              _lhsOoffset =
                  {-# LINE 36 "./AG/CodeGeneration.ag" #-}
                  _tlIoffset
                  {-# LINE 433 "Ag.hs" #-}
              _hdOlabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 437 "Ag.hs" #-}
              _hdOoffset =
                  {-# LINE 35 "./AG/CodeGeneration.ag" #-}
                  _lhsIoffset
                  {-# LINE 441 "Ag.hs" #-}
              _hdOsyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 445 "Ag.hs" #-}
              _hdOuvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 449 "Ag.hs" #-}
              _tlOlabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _hdIlabels
                  {-# LINE 453 "Ag.hs" #-}
              _tlOoffset =
                  {-# LINE 35 "./AG/CodeGeneration.ag" #-}
                  _hdIoffset
                  {-# LINE 457 "Ag.hs" #-}
              _tlOsyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 461 "Ag.hs" #-}
              _tlOuvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 465 "Ag.hs" #-}
              ( _hdIcodes,_hdIenv,_hdIlabels,_hdIoffset,_hdIvars) =
                  (hd_ _hdOlabels _hdOoffset _hdOsyms _hdOuvars )
              ( _tlIcodes,_tlIenv,_tlIlabels,_tlIoffset,_tlIvars) =
                  (tl_ _tlOlabels _tlOoffset _tlOsyms _tlOuvars )
          in  ( _lhsOcodes,_lhsOenv,_lhsOlabels,_lhsOoffset,_lhsOvars)))
sem_Decls_Nil :: T_Decls 
sem_Decls_Nil  =
    (\ _lhsIlabels
       _lhsIoffset
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOenv :: Env
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _lhsOoffset :: Int
              _lhsOcodes =
                  {-# LINE 114 "./AG/CodeGeneration.ag" #-}
                  id
                  {-# LINE 485 "Ag.hs" #-}
              _lhsOenv =
                  {-# LINE 66 "./AG/CodeGeneration.ag" #-}
                  []
                  {-# LINE 489 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 116 "./AG/CodeGeneration.ag" #-}
                  []
                  {-# LINE 493 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 497 "Ag.hs" #-}
              _lhsOoffset =
                  {-# LINE 36 "./AG/CodeGeneration.ag" #-}
                  _lhsIoffset
                  {-# LINE 501 "Ag.hs" #-}
          in  ( _lhsOcodes,_lhsOenv,_lhsOlabels,_lhsOoffset,_lhsOvars)))
-- Exp ---------------------------------------------------------
data Exp  = Add (Exp) (Exp) 
          | Call (Ident) (Exps) 
          | Div (Exp) (Exp) 
          | Eq (Exp) (Exp) 
          | False_ 
          | Gt (Exp) (Exp) 
          | Int (Int) 
          | Lt (Exp) (Exp) 
          | Mul (Exp) (Exp) 
          | Sub (Exp) (Exp) 
          | True_ 
          | Var (Ident) 
-- cata
sem_Exp :: Exp  ->
           T_Exp 
sem_Exp (Add _e1 _e2 )  =
    (sem_Exp_Add (sem_Exp _e1 ) (sem_Exp _e2 ) )
sem_Exp (Call _f _es )  =
    (sem_Exp_Call _f (sem_Exps _es ) )
sem_Exp (Div _e1 _e2 )  =
    (sem_Exp_Div (sem_Exp _e1 ) (sem_Exp _e2 ) )
sem_Exp (Eq _e1 _e2 )  =
    (sem_Exp_Eq (sem_Exp _e1 ) (sem_Exp _e2 ) )
sem_Exp (False_ )  =
    (sem_Exp_False_ )
sem_Exp (Gt _e1 _e2 )  =
    (sem_Exp_Gt (sem_Exp _e1 ) (sem_Exp _e2 ) )
sem_Exp (Int _n )  =
    (sem_Exp_Int _n )
sem_Exp (Lt _e1 _e2 )  =
    (sem_Exp_Lt (sem_Exp _e1 ) (sem_Exp _e2 ) )
sem_Exp (Mul _e1 _e2 )  =
    (sem_Exp_Mul (sem_Exp _e1 ) (sem_Exp _e2 ) )
sem_Exp (Sub _e1 _e2 )  =
    (sem_Exp_Sub (sem_Exp _e1 ) (sem_Exp _e2 ) )
sem_Exp (True_ )  =
    (sem_Exp_True_ )
sem_Exp (Var _x )  =
    (sem_Exp_Var _x )
-- semantic domain
type T_Exp  = ([Label]) ->
              Syms ->
              UsedVars ->
              ( CodeS,([Label]),UsedVars)
data Inh_Exp  = Inh_Exp {labels_Inh_Exp :: [Label],syms_Inh_Exp :: Syms,uvars_Inh_Exp :: UsedVars}
data Syn_Exp  = Syn_Exp {codes_Syn_Exp :: CodeS,labels_Syn_Exp :: [Label],vars_Syn_Exp :: UsedVars}
wrap_Exp :: T_Exp  ->
            Inh_Exp  ->
            Syn_Exp 
wrap_Exp sem (Inh_Exp _lhsIlabels _lhsIsyms _lhsIuvars )  =
    (let ( _lhsOcodes,_lhsOlabels,_lhsOvars) =
             (sem _lhsIlabels _lhsIsyms _lhsIuvars )
     in  (Syn_Exp _lhsOcodes _lhsOlabels _lhsOvars ))
sem_Exp_Add :: T_Exp  ->
               T_Exp  ->
               T_Exp 
sem_Exp_Add e1_ e2_  =
    (\ _lhsIlabels
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _e1Olabels :: ([Label])
              _e1Osyms :: Syms
              _e1Ouvars :: UsedVars
              _e2Olabels :: ([Label])
              _e2Osyms :: Syms
              _e2Ouvars :: UsedVars
              _e1Icodes :: CodeS
              _e1Ilabels :: ([Label])
              _e1Ivars :: UsedVars
              _e2Icodes :: CodeS
              _e2Ilabels :: ([Label])
              _e2Ivars :: UsedVars
              _lhsOcodes =
                  {-# LINE 177 "./AG/CodeGeneration.ag" #-}
                  _e1Icodes . _e2Icodes . add
                  {-# LINE 582 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 116 "./AG/CodeGeneration.ag" #-}
                  _e1Ivars `mergeUsedVars` _e2Ivars
                  {-# LINE 586 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _e2Ilabels
                  {-# LINE 590 "Ag.hs" #-}
              _e1Olabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 594 "Ag.hs" #-}
              _e1Osyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 598 "Ag.hs" #-}
              _e1Ouvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 602 "Ag.hs" #-}
              _e2Olabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _e1Ilabels
                  {-# LINE 606 "Ag.hs" #-}
              _e2Osyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 610 "Ag.hs" #-}
              _e2Ouvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 614 "Ag.hs" #-}
              ( _e1Icodes,_e1Ilabels,_e1Ivars) =
                  (e1_ _e1Olabels _e1Osyms _e1Ouvars )
              ( _e2Icodes,_e2Ilabels,_e2Ivars) =
                  (e2_ _e2Olabels _e2Osyms _e2Ouvars )
          in  ( _lhsOcodes,_lhsOlabels,_lhsOvars)))
sem_Exp_Call :: Ident ->
                T_Exps  ->
                T_Exp 
sem_Exp_Call f_ es_  =
    (\ _lhsIlabels
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _esOlabels :: ([Label])
              _esOsyms :: Syms
              _esOuvars :: UsedVars
              _esIcodes :: CodeS
              _esIlabels :: ([Label])
              _esIvars :: UsedVars
              _lhsOcodes =
                  {-# LINE 175 "./AG/CodeGeneration.ag" #-}
                  _esIcodes . call f_ _lhsIsyms
                  {-# LINE 639 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 176 "./AG/CodeGeneration.ag" #-}
                  _esIvars
                  {-# LINE 643 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _esIlabels
                  {-# LINE 647 "Ag.hs" #-}
              _esOlabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 651 "Ag.hs" #-}
              _esOsyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 655 "Ag.hs" #-}
              _esOuvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 659 "Ag.hs" #-}
              ( _esIcodes,_esIlabels,_esIvars) =
                  (es_ _esOlabels _esOsyms _esOuvars )
          in  ( _lhsOcodes,_lhsOlabels,_lhsOvars)))
sem_Exp_Div :: T_Exp  ->
               T_Exp  ->
               T_Exp 
sem_Exp_Div e1_ e2_  =
    (\ _lhsIlabels
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _e1Olabels :: ([Label])
              _e1Osyms :: Syms
              _e1Ouvars :: UsedVars
              _e2Olabels :: ([Label])
              _e2Osyms :: Syms
              _e2Ouvars :: UsedVars
              _e1Icodes :: CodeS
              _e1Ilabels :: ([Label])
              _e1Ivars :: UsedVars
              _e2Icodes :: CodeS
              _e2Ilabels :: ([Label])
              _e2Ivars :: UsedVars
              _lhsOcodes =
                  {-# LINE 180 "./AG/CodeGeneration.ag" #-}
                  _e1Icodes . _e2Icodes . div
                  {-# LINE 688 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 116 "./AG/CodeGeneration.ag" #-}
                  _e1Ivars `mergeUsedVars` _e2Ivars
                  {-# LINE 692 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _e2Ilabels
                  {-# LINE 696 "Ag.hs" #-}
              _e1Olabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 700 "Ag.hs" #-}
              _e1Osyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 704 "Ag.hs" #-}
              _e1Ouvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 708 "Ag.hs" #-}
              _e2Olabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _e1Ilabels
                  {-# LINE 712 "Ag.hs" #-}
              _e2Osyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 716 "Ag.hs" #-}
              _e2Ouvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 720 "Ag.hs" #-}
              ( _e1Icodes,_e1Ilabels,_e1Ivars) =
                  (e1_ _e1Olabels _e1Osyms _e1Ouvars )
              ( _e2Icodes,_e2Ilabels,_e2Ivars) =
                  (e2_ _e2Olabels _e2Osyms _e2Ouvars )
          in  ( _lhsOcodes,_lhsOlabels,_lhsOvars)))
sem_Exp_Eq :: T_Exp  ->
              T_Exp  ->
              T_Exp 
sem_Exp_Eq e1_ e2_  =
    (\ _lhsIlabels
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _e1Olabels :: ([Label])
              _e1Osyms :: Syms
              _e1Ouvars :: UsedVars
              _e2Olabels :: ([Label])
              _e2Osyms :: Syms
              _e2Ouvars :: UsedVars
              _e1Icodes :: CodeS
              _e1Ilabels :: ([Label])
              _e1Ivars :: UsedVars
              _e2Icodes :: CodeS
              _e2Ilabels :: ([Label])
              _e2Ivars :: UsedVars
              _lhsOcodes =
                  {-# LINE 182 "./AG/CodeGeneration.ag" #-}
                  _e1Icodes . _e2Icodes . eq
                  {-# LINE 751 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 116 "./AG/CodeGeneration.ag" #-}
                  _e1Ivars `mergeUsedVars` _e2Ivars
                  {-# LINE 755 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _e2Ilabels
                  {-# LINE 759 "Ag.hs" #-}
              _e1Olabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 763 "Ag.hs" #-}
              _e1Osyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 767 "Ag.hs" #-}
              _e1Ouvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 771 "Ag.hs" #-}
              _e2Olabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _e1Ilabels
                  {-# LINE 775 "Ag.hs" #-}
              _e2Osyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 779 "Ag.hs" #-}
              _e2Ouvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 783 "Ag.hs" #-}
              ( _e1Icodes,_e1Ilabels,_e1Ivars) =
                  (e1_ _e1Olabels _e1Osyms _e1Ouvars )
              ( _e2Icodes,_e2Ilabels,_e2Ivars) =
                  (e2_ _e2Olabels _e2Osyms _e2Ouvars )
          in  ( _lhsOcodes,_lhsOlabels,_lhsOvars)))
sem_Exp_False_ :: T_Exp 
sem_Exp_False_  =
    (\ _lhsIlabels
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _lhsOcodes =
                  {-# LINE 170 "./AG/CodeGeneration.ag" #-}
                  ldc 0
                  {-# LINE 800 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 116 "./AG/CodeGeneration.ag" #-}
                  []
                  {-# LINE 804 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 808 "Ag.hs" #-}
          in  ( _lhsOcodes,_lhsOlabels,_lhsOvars)))
sem_Exp_Gt :: T_Exp  ->
              T_Exp  ->
              T_Exp 
sem_Exp_Gt e1_ e2_  =
    (\ _lhsIlabels
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _e1Olabels :: ([Label])
              _e1Osyms :: Syms
              _e1Ouvars :: UsedVars
              _e2Olabels :: ([Label])
              _e2Osyms :: Syms
              _e2Ouvars :: UsedVars
              _e1Icodes :: CodeS
              _e1Ilabels :: ([Label])
              _e1Ivars :: UsedVars
              _e2Icodes :: CodeS
              _e2Ilabels :: ([Label])
              _e2Ivars :: UsedVars
              _lhsOcodes =
                  {-# LINE 183 "./AG/CodeGeneration.ag" #-}
                  _e1Icodes . _e2Icodes . gt
                  {-# LINE 835 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 116 "./AG/CodeGeneration.ag" #-}
                  _e1Ivars `mergeUsedVars` _e2Ivars
                  {-# LINE 839 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _e2Ilabels
                  {-# LINE 843 "Ag.hs" #-}
              _e1Olabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 847 "Ag.hs" #-}
              _e1Osyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 851 "Ag.hs" #-}
              _e1Ouvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 855 "Ag.hs" #-}
              _e2Olabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _e1Ilabels
                  {-# LINE 859 "Ag.hs" #-}
              _e2Osyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 863 "Ag.hs" #-}
              _e2Ouvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 867 "Ag.hs" #-}
              ( _e1Icodes,_e1Ilabels,_e1Ivars) =
                  (e1_ _e1Olabels _e1Osyms _e1Ouvars )
              ( _e2Icodes,_e2Ilabels,_e2Ivars) =
                  (e2_ _e2Olabels _e2Osyms _e2Ouvars )
          in  ( _lhsOcodes,_lhsOlabels,_lhsOvars)))
sem_Exp_Int :: Int ->
               T_Exp 
sem_Exp_Int n_  =
    (\ _lhsIlabels
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _lhsOcodes =
                  {-# LINE 169 "./AG/CodeGeneration.ag" #-}
                  ldc n_
                  {-# LINE 885 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 116 "./AG/CodeGeneration.ag" #-}
                  []
                  {-# LINE 889 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 893 "Ag.hs" #-}
          in  ( _lhsOcodes,_lhsOlabels,_lhsOvars)))
sem_Exp_Lt :: T_Exp  ->
              T_Exp  ->
              T_Exp 
sem_Exp_Lt e1_ e2_  =
    (\ _lhsIlabels
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _e1Olabels :: ([Label])
              _e1Osyms :: Syms
              _e1Ouvars :: UsedVars
              _e2Olabels :: ([Label])
              _e2Osyms :: Syms
              _e2Ouvars :: UsedVars
              _e1Icodes :: CodeS
              _e1Ilabels :: ([Label])
              _e1Ivars :: UsedVars
              _e2Icodes :: CodeS
              _e2Ilabels :: ([Label])
              _e2Ivars :: UsedVars
              _lhsOcodes =
                  {-# LINE 181 "./AG/CodeGeneration.ag" #-}
                  _e1Icodes . _e2Icodes . lt
                  {-# LINE 920 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 116 "./AG/CodeGeneration.ag" #-}
                  _e1Ivars `mergeUsedVars` _e2Ivars
                  {-# LINE 924 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _e2Ilabels
                  {-# LINE 928 "Ag.hs" #-}
              _e1Olabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 932 "Ag.hs" #-}
              _e1Osyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 936 "Ag.hs" #-}
              _e1Ouvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 940 "Ag.hs" #-}
              _e2Olabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _e1Ilabels
                  {-# LINE 944 "Ag.hs" #-}
              _e2Osyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 948 "Ag.hs" #-}
              _e2Ouvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 952 "Ag.hs" #-}
              ( _e1Icodes,_e1Ilabels,_e1Ivars) =
                  (e1_ _e1Olabels _e1Osyms _e1Ouvars )
              ( _e2Icodes,_e2Ilabels,_e2Ivars) =
                  (e2_ _e2Olabels _e2Osyms _e2Ouvars )
          in  ( _lhsOcodes,_lhsOlabels,_lhsOvars)))
sem_Exp_Mul :: T_Exp  ->
               T_Exp  ->
               T_Exp 
sem_Exp_Mul e1_ e2_  =
    (\ _lhsIlabels
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _e1Olabels :: ([Label])
              _e1Osyms :: Syms
              _e1Ouvars :: UsedVars
              _e2Olabels :: ([Label])
              _e2Osyms :: Syms
              _e2Ouvars :: UsedVars
              _e1Icodes :: CodeS
              _e1Ilabels :: ([Label])
              _e1Ivars :: UsedVars
              _e2Icodes :: CodeS
              _e2Ilabels :: ([Label])
              _e2Ivars :: UsedVars
              _lhsOcodes =
                  {-# LINE 179 "./AG/CodeGeneration.ag" #-}
                  _e1Icodes . _e2Icodes . mul
                  {-# LINE 983 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 116 "./AG/CodeGeneration.ag" #-}
                  _e1Ivars `mergeUsedVars` _e2Ivars
                  {-# LINE 987 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _e2Ilabels
                  {-# LINE 991 "Ag.hs" #-}
              _e1Olabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 995 "Ag.hs" #-}
              _e1Osyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 999 "Ag.hs" #-}
              _e1Ouvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 1003 "Ag.hs" #-}
              _e2Olabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _e1Ilabels
                  {-# LINE 1007 "Ag.hs" #-}
              _e2Osyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 1011 "Ag.hs" #-}
              _e2Ouvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 1015 "Ag.hs" #-}
              ( _e1Icodes,_e1Ilabels,_e1Ivars) =
                  (e1_ _e1Olabels _e1Osyms _e1Ouvars )
              ( _e2Icodes,_e2Ilabels,_e2Ivars) =
                  (e2_ _e2Olabels _e2Osyms _e2Ouvars )
          in  ( _lhsOcodes,_lhsOlabels,_lhsOvars)))
sem_Exp_Sub :: T_Exp  ->
               T_Exp  ->
               T_Exp 
sem_Exp_Sub e1_ e2_  =
    (\ _lhsIlabels
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _e1Olabels :: ([Label])
              _e1Osyms :: Syms
              _e1Ouvars :: UsedVars
              _e2Olabels :: ([Label])
              _e2Osyms :: Syms
              _e2Ouvars :: UsedVars
              _e1Icodes :: CodeS
              _e1Ilabels :: ([Label])
              _e1Ivars :: UsedVars
              _e2Icodes :: CodeS
              _e2Ilabels :: ([Label])
              _e2Ivars :: UsedVars
              _lhsOcodes =
                  {-# LINE 178 "./AG/CodeGeneration.ag" #-}
                  _e1Icodes . _e2Icodes . sub
                  {-# LINE 1046 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 116 "./AG/CodeGeneration.ag" #-}
                  _e1Ivars `mergeUsedVars` _e2Ivars
                  {-# LINE 1050 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _e2Ilabels
                  {-# LINE 1054 "Ag.hs" #-}
              _e1Olabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 1058 "Ag.hs" #-}
              _e1Osyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 1062 "Ag.hs" #-}
              _e1Ouvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 1066 "Ag.hs" #-}
              _e2Olabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _e1Ilabels
                  {-# LINE 1070 "Ag.hs" #-}
              _e2Osyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 1074 "Ag.hs" #-}
              _e2Ouvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 1078 "Ag.hs" #-}
              ( _e1Icodes,_e1Ilabels,_e1Ivars) =
                  (e1_ _e1Olabels _e1Osyms _e1Ouvars )
              ( _e2Icodes,_e2Ilabels,_e2Ivars) =
                  (e2_ _e2Olabels _e2Osyms _e2Ouvars )
          in  ( _lhsOcodes,_lhsOlabels,_lhsOvars)))
sem_Exp_True_ :: T_Exp 
sem_Exp_True_  =
    (\ _lhsIlabels
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _lhsOcodes =
                  {-# LINE 171 "./AG/CodeGeneration.ag" #-}
                  ldc 1
                  {-# LINE 1095 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 116 "./AG/CodeGeneration.ag" #-}
                  []
                  {-# LINE 1099 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 1103 "Ag.hs" #-}
          in  ( _lhsOcodes,_lhsOlabels,_lhsOvars)))
sem_Exp_Var :: Ident ->
               T_Exp 
sem_Exp_Var x_  =
    (\ _lhsIlabels
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _lhsOcodes =
                  {-# LINE 172 "./AG/CodeGeneration.ag" #-}
                  get x_ _lhsIsyms _lhsIuvars
                  {-# LINE 1117 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 174 "./AG/CodeGeneration.ag" #-}
                  incrUsage x_ [] 1
                  {-# LINE 1121 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 1125 "Ag.hs" #-}
          in  ( _lhsOcodes,_lhsOlabels,_lhsOvars)))
-- Exps --------------------------------------------------------
type Exps  = [(Exp)]
-- cata
sem_Exps :: Exps  ->
            T_Exps 
sem_Exps list  =
    (Prelude.foldr sem_Exps_Cons sem_Exps_Nil (Prelude.map sem_Exp list) )
-- semantic domain
type T_Exps  = ([Label]) ->
               Syms ->
               UsedVars ->
               ( CodeS,([Label]),UsedVars)
data Inh_Exps  = Inh_Exps {labels_Inh_Exps :: [Label],syms_Inh_Exps :: Syms,uvars_Inh_Exps :: UsedVars}
data Syn_Exps  = Syn_Exps {codes_Syn_Exps :: CodeS,labels_Syn_Exps :: [Label],vars_Syn_Exps :: UsedVars}
wrap_Exps :: T_Exps  ->
             Inh_Exps  ->
             Syn_Exps 
wrap_Exps sem (Inh_Exps _lhsIlabels _lhsIsyms _lhsIuvars )  =
    (let ( _lhsOcodes,_lhsOlabels,_lhsOvars) =
             (sem _lhsIlabels _lhsIsyms _lhsIuvars )
     in  (Syn_Exps _lhsOcodes _lhsOlabels _lhsOvars ))
sem_Exps_Cons :: T_Exp  ->
                 T_Exps  ->
                 T_Exps 
sem_Exps_Cons hd_ tl_  =
    (\ _lhsIlabels
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _hdOlabels :: ([Label])
              _hdOsyms :: Syms
              _hdOuvars :: UsedVars
              _tlOlabels :: ([Label])
              _tlOsyms :: Syms
              _tlOuvars :: UsedVars
              _hdIcodes :: CodeS
              _hdIlabels :: ([Label])
              _hdIvars :: UsedVars
              _tlIcodes :: CodeS
              _tlIlabels :: ([Label])
              _tlIvars :: UsedVars
              _lhsOcodes =
                  {-# LINE 114 "./AG/CodeGeneration.ag" #-}
                  _hdIcodes . _tlIcodes
                  {-# LINE 1173 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 116 "./AG/CodeGeneration.ag" #-}
                  _hdIvars `mergeUsedVars` _tlIvars
                  {-# LINE 1177 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _tlIlabels
                  {-# LINE 1181 "Ag.hs" #-}
              _hdOlabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 1185 "Ag.hs" #-}
              _hdOsyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 1189 "Ag.hs" #-}
              _hdOuvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 1193 "Ag.hs" #-}
              _tlOlabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _hdIlabels
                  {-# LINE 1197 "Ag.hs" #-}
              _tlOsyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 1201 "Ag.hs" #-}
              _tlOuvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 1205 "Ag.hs" #-}
              ( _hdIcodes,_hdIlabels,_hdIvars) =
                  (hd_ _hdOlabels _hdOsyms _hdOuvars )
              ( _tlIcodes,_tlIlabels,_tlIvars) =
                  (tl_ _tlOlabels _tlOsyms _tlOuvars )
          in  ( _lhsOcodes,_lhsOlabels,_lhsOvars)))
sem_Exps_Nil :: T_Exps 
sem_Exps_Nil  =
    (\ _lhsIlabels
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _lhsOcodes =
                  {-# LINE 114 "./AG/CodeGeneration.ag" #-}
                  id
                  {-# LINE 1222 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 116 "./AG/CodeGeneration.ag" #-}
                  []
                  {-# LINE 1226 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 1230 "Ag.hs" #-}
          in  ( _lhsOcodes,_lhsOlabels,_lhsOvars)))
-- Prog --------------------------------------------------------
data Prog  = TopLevelDecls (Decls) 
-- cata
sem_Prog :: Prog  ->
            T_Prog 
sem_Prog (TopLevelDecls _ds )  =
    (sem_Prog_TopLevelDecls (sem_Decls _ds ) )
-- semantic domain
type T_Prog  = ( Code)
data Inh_Prog  = Inh_Prog {}
data Syn_Prog  = Syn_Prog {code_Syn_Prog :: Code}
wrap_Prog :: T_Prog  ->
             Inh_Prog  ->
             Syn_Prog 
wrap_Prog sem (Inh_Prog )  =
    (let ( _lhsOcode) =
             (sem )
     in  (Syn_Prog _lhsOcode ))
sem_Prog_TopLevelDecls :: T_Decls  ->
                          T_Prog 
sem_Prog_TopLevelDecls ds_  =
    (let _dsOlabels :: ([Label])
         _dsOoffset :: Int
         _dsOsyms :: Syms
         _dsOuvars :: UsedVars
         _lhsOcode :: Code
         _dsIcodes :: CodeS
         _dsIenv :: Env
         _dsIlabels :: ([Label])
         _dsIoffset :: Int
         _dsIvars :: UsedVars
         _dsOlabels =
             {-# LINE 17 "./AG/CodeGeneration.ag" #-}
             [0 ..]
             {-# LINE 1266 "Ag.hs" #-}
         _dsOoffset =
             {-# LINE 38 "./AG/CodeGeneration.ag" #-}
             1
             {-# LINE 1270 "Ag.hs" #-}
         _dsOsyms =
             {-# LINE 99 "./AG/CodeGeneration.ag" #-}
             [_dsIenv]
             {-# LINE 1274 "Ag.hs" #-}
         _codes =
             {-# LINE 121 "./AG/CodeGeneration.ag" #-}
             _dsIcodes .
             enter _dsIenv .
             call "main" [_dsIenv] .
             ajs (- 1) .
             exit _dsIenv
             {-# LINE 1282 "Ag.hs" #-}
         _dsOuvars =
             {-# LINE 126 "./AG/CodeGeneration.ag" #-}
             []
             {-# LINE 1286 "Ag.hs" #-}
         _lhsOcode =
             {-# LINE 188 "./AG/CodeGeneration.ag" #-}
             Code (_codes     [])
             {-# LINE 1290 "Ag.hs" #-}
         ( _dsIcodes,_dsIenv,_dsIlabels,_dsIoffset,_dsIvars) =
             (ds_ _dsOlabels _dsOoffset _dsOsyms _dsOuvars )
     in  ( _lhsOcode))
-- Stmt --------------------------------------------------------
data Stmt  = Assign (Ident) (Exp) 
           | Block (Stmts) 
           | Call_ (Ident) (Exps) 
           | Decl (Decl) 
           | Empty 
           | If (Exp) (Stmt) (Stmt) 
           | Print (Exp) 
           | Return (Exp) 
-- cata
sem_Stmt :: Stmt  ->
            T_Stmt 
sem_Stmt (Assign _x _e )  =
    (sem_Stmt_Assign _x (sem_Exp _e ) )
sem_Stmt (Block _b )  =
    (sem_Stmt_Block (sem_Stmts _b ) )
sem_Stmt (Call_ _f _es )  =
    (sem_Stmt_Call_ _f (sem_Exps _es ) )
sem_Stmt (Decl _d )  =
    (sem_Stmt_Decl (sem_Decl _d ) )
sem_Stmt (Empty )  =
    (sem_Stmt_Empty )
sem_Stmt (If _e _s1 _s2 )  =
    (sem_Stmt_If (sem_Exp _e ) (sem_Stmt _s1 ) (sem_Stmt _s2 ) )
sem_Stmt (Print _e )  =
    (sem_Stmt_Print (sem_Exp _e ) )
sem_Stmt (Return _e )  =
    (sem_Stmt_Return (sem_Exp _e ) )
-- semantic domain
type T_Stmt  = ([Label]) ->
               Int ->
               Syms ->
               UsedVars ->
               ( CodeS,Env,([Label]),Int,UsedVars)
data Inh_Stmt  = Inh_Stmt {labels_Inh_Stmt :: [Label],offset_Inh_Stmt :: Int,syms_Inh_Stmt :: Syms,uvars_Inh_Stmt :: UsedVars}
data Syn_Stmt  = Syn_Stmt {codes_Syn_Stmt :: CodeS,env_Syn_Stmt :: Env,labels_Syn_Stmt :: [Label],offset_Syn_Stmt :: Int,vars_Syn_Stmt :: UsedVars}
wrap_Stmt :: T_Stmt  ->
             Inh_Stmt  ->
             Syn_Stmt 
wrap_Stmt sem (Inh_Stmt _lhsIlabels _lhsIoffset _lhsIsyms _lhsIuvars )  =
    (let ( _lhsOcodes,_lhsOenv,_lhsOlabels,_lhsOoffset,_lhsOvars) =
             (sem _lhsIlabels _lhsIoffset _lhsIsyms _lhsIuvars )
     in  (Syn_Stmt _lhsOcodes _lhsOenv _lhsOlabels _lhsOoffset _lhsOvars ))
sem_Stmt_Assign :: Ident ->
                   T_Exp  ->
                   T_Stmt 
sem_Stmt_Assign x_ e_  =
    (\ _lhsIlabels
       _lhsIoffset
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _lhsOenv :: Env
              _lhsOlabels :: ([Label])
              _lhsOoffset :: Int
              _eOlabels :: ([Label])
              _eOsyms :: Syms
              _eOuvars :: UsedVars
              _eIcodes :: CodeS
              _eIlabels :: ([Label])
              _eIvars :: UsedVars
              _lhsOcodes =
                  {-# LINE 146 "./AG/CodeGeneration.ag" #-}
                  _eIcodes . set x_ _lhsIsyms _lhsIuvars
                  {-# LINE 1359 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 148 "./AG/CodeGeneration.ag" #-}
                  incrUsage x_ _eIvars 1
                  {-# LINE 1363 "Ag.hs" #-}
              _lhsOenv =
                  {-# LINE 66 "./AG/CodeGeneration.ag" #-}
                  []
                  {-# LINE 1367 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _eIlabels
                  {-# LINE 1371 "Ag.hs" #-}
              _lhsOoffset =
                  {-# LINE 36 "./AG/CodeGeneration.ag" #-}
                  _lhsIoffset
                  {-# LINE 1375 "Ag.hs" #-}
              _eOlabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 1379 "Ag.hs" #-}
              _eOsyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 1383 "Ag.hs" #-}
              _eOuvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 1387 "Ag.hs" #-}
              ( _eIcodes,_eIlabels,_eIvars) =
                  (e_ _eOlabels _eOsyms _eOuvars )
          in  ( _lhsOcodes,_lhsOenv,_lhsOlabels,_lhsOoffset,_lhsOvars)))
sem_Stmt_Block :: T_Stmts  ->
                  T_Stmt 
sem_Stmt_Block b_  =
    (\ _lhsIlabels
       _lhsIoffset
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOoffset :: Int
              _lhsOenv :: Env
              _bOsyms :: Syms
              _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _bOlabels :: ([Label])
              _bOoffset :: Int
              _bOuvars :: UsedVars
              _bIcodes :: CodeS
              _bIenv :: Env
              _bIlabels :: ([Label])
              _bIoffset :: Int
              _bIvars :: UsedVars
              _lhsOoffset =
                  {-# LINE 41 "./AG/CodeGeneration.ag" #-}
                  _lhsIoffset
                  {-# LINE 1415 "Ag.hs" #-}
              _lhsOenv =
                  {-# LINE 71 "./AG/CodeGeneration.ag" #-}
                  []
                  {-# LINE 1419 "Ag.hs" #-}
              _bOsyms =
                  {-# LINE 101 "./AG/CodeGeneration.ag" #-}
                  let local : global = _lhsIsyms
                  in  (local ++ _bIenv) : global
                  {-# LINE 1424 "Ag.hs" #-}
              _lhsOcodes =
                  {-# LINE 164 "./AG/CodeGeneration.ag" #-}
                  enter _bIenv .
                  _bIcodes .
                  exit _bIenv
                  {-# LINE 1430 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 167 "./AG/CodeGeneration.ag" #-}
                  _bIvars
                  {-# LINE 1434 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _bIlabels
                  {-# LINE 1438 "Ag.hs" #-}
              _bOlabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 1442 "Ag.hs" #-}
              _bOoffset =
                  {-# LINE 35 "./AG/CodeGeneration.ag" #-}
                  _lhsIoffset
                  {-# LINE 1446 "Ag.hs" #-}
              _bOuvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 1450 "Ag.hs" #-}
              ( _bIcodes,_bIenv,_bIlabels,_bIoffset,_bIvars) =
                  (b_ _bOlabels _bOoffset _bOsyms _bOuvars )
          in  ( _lhsOcodes,_lhsOenv,_lhsOlabels,_lhsOoffset,_lhsOvars)))
sem_Stmt_Call_ :: Ident ->
                  T_Exps  ->
                  T_Stmt 
sem_Stmt_Call_ f_ es_  =
    (\ _lhsIlabels
       _lhsIoffset
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _lhsOenv :: Env
              _lhsOlabels :: ([Label])
              _lhsOoffset :: Int
              _esOlabels :: ([Label])
              _esOsyms :: Syms
              _esOuvars :: UsedVars
              _esIcodes :: CodeS
              _esIlabels :: ([Label])
              _esIvars :: UsedVars
              _lhsOcodes =
                  {-# LINE 149 "./AG/CodeGeneration.ag" #-}
                  _esIcodes .
                  call f_ _lhsIsyms .
                  ajs (- 1)
                  {-# LINE 1478 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 152 "./AG/CodeGeneration.ag" #-}
                  _esIvars
                  {-# LINE 1482 "Ag.hs" #-}
              _lhsOenv =
                  {-# LINE 66 "./AG/CodeGeneration.ag" #-}
                  []
                  {-# LINE 1486 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _esIlabels
                  {-# LINE 1490 "Ag.hs" #-}
              _lhsOoffset =
                  {-# LINE 36 "./AG/CodeGeneration.ag" #-}
                  _lhsIoffset
                  {-# LINE 1494 "Ag.hs" #-}
              _esOlabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 1498 "Ag.hs" #-}
              _esOsyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 1502 "Ag.hs" #-}
              _esOuvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 1506 "Ag.hs" #-}
              ( _esIcodes,_esIlabels,_esIvars) =
                  (es_ _esOlabels _esOsyms _esOuvars )
          in  ( _lhsOcodes,_lhsOenv,_lhsOlabels,_lhsOoffset,_lhsOvars)))
sem_Stmt_Decl :: T_Decl  ->
                 T_Stmt 
sem_Stmt_Decl d_  =
    (\ _lhsIlabels
       _lhsIoffset
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOenv :: Env
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _lhsOoffset :: Int
              _dOlabels :: ([Label])
              _dOoffset :: Int
              _dOsyms :: Syms
              _dOuvars :: UsedVars
              _dIcodes :: CodeS
              _dIenv :: Env
              _dIlabels :: ([Label])
              _dIoffset :: Int
              _dIvars :: UsedVars
              _lhsOcodes =
                  {-# LINE 114 "./AG/CodeGeneration.ag" #-}
                  _dIcodes
                  {-# LINE 1534 "Ag.hs" #-}
              _lhsOenv =
                  {-# LINE 66 "./AG/CodeGeneration.ag" #-}
                  _dIenv
                  {-# LINE 1538 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 116 "./AG/CodeGeneration.ag" #-}
                  _dIvars
                  {-# LINE 1542 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _dIlabels
                  {-# LINE 1546 "Ag.hs" #-}
              _lhsOoffset =
                  {-# LINE 36 "./AG/CodeGeneration.ag" #-}
                  _dIoffset
                  {-# LINE 1550 "Ag.hs" #-}
              _dOlabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 1554 "Ag.hs" #-}
              _dOoffset =
                  {-# LINE 35 "./AG/CodeGeneration.ag" #-}
                  _lhsIoffset
                  {-# LINE 1558 "Ag.hs" #-}
              _dOsyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 1562 "Ag.hs" #-}
              _dOuvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 1566 "Ag.hs" #-}
              ( _dIcodes,_dIenv,_dIlabels,_dIoffset,_dIvars) =
                  (d_ _dOlabels _dOoffset _dOsyms _dOuvars )
          in  ( _lhsOcodes,_lhsOenv,_lhsOlabels,_lhsOoffset,_lhsOvars)))
sem_Stmt_Empty :: T_Stmt 
sem_Stmt_Empty  =
    (\ _lhsIlabels
       _lhsIoffset
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOenv :: Env
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _lhsOoffset :: Int
              _lhsOcodes =
                  {-# LINE 114 "./AG/CodeGeneration.ag" #-}
                  id
                  {-# LINE 1584 "Ag.hs" #-}
              _lhsOenv =
                  {-# LINE 66 "./AG/CodeGeneration.ag" #-}
                  []
                  {-# LINE 1588 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 116 "./AG/CodeGeneration.ag" #-}
                  []
                  {-# LINE 1592 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 1596 "Ag.hs" #-}
              _lhsOoffset =
                  {-# LINE 36 "./AG/CodeGeneration.ag" #-}
                  _lhsIoffset
                  {-# LINE 1600 "Ag.hs" #-}
          in  ( _lhsOcodes,_lhsOenv,_lhsOlabels,_lhsOoffset,_lhsOvars)))
sem_Stmt_If :: T_Exp  ->
               T_Stmt  ->
               T_Stmt  ->
               T_Stmt 
sem_Stmt_If e_ s1_ s2_  =
    (\ _lhsIlabels
       _lhsIoffset
       _lhsIsyms
       _lhsIuvars ->
         (let _eOlabels :: ([Label])
              _lhsOcodes :: CodeS
              _lhsOenv :: Env
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _lhsOoffset :: Int
              _eOsyms :: Syms
              _eOuvars :: UsedVars
              _s1Olabels :: ([Label])
              _s1Ooffset :: Int
              _s1Osyms :: Syms
              _s1Ouvars :: UsedVars
              _s2Olabels :: ([Label])
              _s2Ooffset :: Int
              _s2Osyms :: Syms
              _s2Ouvars :: UsedVars
              _eIcodes :: CodeS
              _eIlabels :: ([Label])
              _eIvars :: UsedVars
              _s1Icodes :: CodeS
              _s1Ienv :: Env
              _s1Ilabels :: ([Label])
              _s1Ioffset :: Int
              _s1Ivars :: UsedVars
              _s2Icodes :: CodeS
              _s2Ienv :: Env
              _s2Ilabels :: ([Label])
              _s2Ioffset :: Int
              _s2Ivars :: UsedVars
              _elseLabel =
                  {-# LINE 23 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels !! 0
                  {-# LINE 1643 "Ag.hs" #-}
              _fiLabel =
                  {-# LINE 24 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels !! 1
                  {-# LINE 1647 "Ag.hs" #-}
              _eOlabels =
                  {-# LINE 25 "./AG/CodeGeneration.ag" #-}
                  drop 2 _lhsIlabels
                  {-# LINE 1651 "Ag.hs" #-}
              _lhsOcodes =
                  {-# LINE 157 "./AG/CodeGeneration.ag" #-}
                  _eIcodes .
                  brf _elseLabel     .
                  _s1Icodes .
                  bra _fiLabel     .
                  label _elseLabel     .
                    _s2Icodes .
                  label _fiLabel
                  {-# LINE 1661 "Ag.hs" #-}
              _lhsOenv =
                  {-# LINE 66 "./AG/CodeGeneration.ag" #-}
                  _s1Ienv ++ _s2Ienv
                  {-# LINE 1665 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 116 "./AG/CodeGeneration.ag" #-}
                  _eIvars `mergeUsedVars` _s1Ivars `mergeUsedVars` _s2Ivars
                  {-# LINE 1669 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _s2Ilabels
                  {-# LINE 1673 "Ag.hs" #-}
              _lhsOoffset =
                  {-# LINE 36 "./AG/CodeGeneration.ag" #-}
                  _s2Ioffset
                  {-# LINE 1677 "Ag.hs" #-}
              _eOsyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 1681 "Ag.hs" #-}
              _eOuvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 1685 "Ag.hs" #-}
              _s1Olabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _eIlabels
                  {-# LINE 1689 "Ag.hs" #-}
              _s1Ooffset =
                  {-# LINE 35 "./AG/CodeGeneration.ag" #-}
                  _lhsIoffset
                  {-# LINE 1693 "Ag.hs" #-}
              _s1Osyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 1697 "Ag.hs" #-}
              _s1Ouvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 1701 "Ag.hs" #-}
              _s2Olabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _s1Ilabels
                  {-# LINE 1705 "Ag.hs" #-}
              _s2Ooffset =
                  {-# LINE 35 "./AG/CodeGeneration.ag" #-}
                  _s1Ioffset
                  {-# LINE 1709 "Ag.hs" #-}
              _s2Osyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 1713 "Ag.hs" #-}
              _s2Ouvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 1717 "Ag.hs" #-}
              ( _eIcodes,_eIlabels,_eIvars) =
                  (e_ _eOlabels _eOsyms _eOuvars )
              ( _s1Icodes,_s1Ienv,_s1Ilabels,_s1Ioffset,_s1Ivars) =
                  (s1_ _s1Olabels _s1Ooffset _s1Osyms _s1Ouvars )
              ( _s2Icodes,_s2Ienv,_s2Ilabels,_s2Ioffset,_s2Ivars) =
                  (s2_ _s2Olabels _s2Ooffset _s2Osyms _s2Ouvars )
          in  ( _lhsOcodes,_lhsOenv,_lhsOlabels,_lhsOoffset,_lhsOvars)))
sem_Stmt_Print :: T_Exp  ->
                  T_Stmt 
sem_Stmt_Print e_  =
    (\ _lhsIlabels
       _lhsIoffset
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _lhsOenv :: Env
              _lhsOlabels :: ([Label])
              _lhsOoffset :: Int
              _eOlabels :: ([Label])
              _eOsyms :: Syms
              _eOuvars :: UsedVars
              _eIcodes :: CodeS
              _eIlabels :: ([Label])
              _eIvars :: UsedVars
              _lhsOcodes =
                  {-# LINE 153 "./AG/CodeGeneration.ag" #-}
                  _eIcodes . trap 0
                  {-# LINE 1746 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 154 "./AG/CodeGeneration.ag" #-}
                  _eIvars
                  {-# LINE 1750 "Ag.hs" #-}
              _lhsOenv =
                  {-# LINE 66 "./AG/CodeGeneration.ag" #-}
                  []
                  {-# LINE 1754 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _eIlabels
                  {-# LINE 1758 "Ag.hs" #-}
              _lhsOoffset =
                  {-# LINE 36 "./AG/CodeGeneration.ag" #-}
                  _lhsIoffset
                  {-# LINE 1762 "Ag.hs" #-}
              _eOlabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 1766 "Ag.hs" #-}
              _eOsyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 1770 "Ag.hs" #-}
              _eOuvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 1774 "Ag.hs" #-}
              ( _eIcodes,_eIlabels,_eIvars) =
                  (e_ _eOlabels _eOsyms _eOuvars )
          in  ( _lhsOcodes,_lhsOenv,_lhsOlabels,_lhsOoffset,_lhsOvars)))
sem_Stmt_Return :: T_Exp  ->
                   T_Stmt 
sem_Stmt_Return e_  =
    (\ _lhsIlabels
       _lhsIoffset
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOvars :: UsedVars
              _lhsOenv :: Env
              _lhsOlabels :: ([Label])
              _lhsOoffset :: Int
              _eOlabels :: ([Label])
              _eOsyms :: Syms
              _eOuvars :: UsedVars
              _eIcodes :: CodeS
              _eIlabels :: ([Label])
              _eIvars :: UsedVars
              _lhsOcodes =
                  {-# LINE 155 "./AG/CodeGeneration.ag" #-}
                  _eIcodes . return_ _lhsIsyms
                  {-# LINE 1799 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 156 "./AG/CodeGeneration.ag" #-}
                  _eIvars
                  {-# LINE 1803 "Ag.hs" #-}
              _lhsOenv =
                  {-# LINE 66 "./AG/CodeGeneration.ag" #-}
                  []
                  {-# LINE 1807 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _eIlabels
                  {-# LINE 1811 "Ag.hs" #-}
              _lhsOoffset =
                  {-# LINE 36 "./AG/CodeGeneration.ag" #-}
                  _lhsIoffset
                  {-# LINE 1815 "Ag.hs" #-}
              _eOlabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 1819 "Ag.hs" #-}
              _eOsyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 1823 "Ag.hs" #-}
              _eOuvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 1827 "Ag.hs" #-}
              ( _eIcodes,_eIlabels,_eIvars) =
                  (e_ _eOlabels _eOsyms _eOuvars )
          in  ( _lhsOcodes,_lhsOenv,_lhsOlabels,_lhsOoffset,_lhsOvars)))
-- Stmts -------------------------------------------------------
type Stmts  = [(Stmt)]
-- cata
sem_Stmts :: Stmts  ->
             T_Stmts 
sem_Stmts list  =
    (Prelude.foldr sem_Stmts_Cons sem_Stmts_Nil (Prelude.map sem_Stmt list) )
-- semantic domain
type T_Stmts  = ([Label]) ->
                Int ->
                Syms ->
                UsedVars ->
                ( CodeS,Env,([Label]),Int,UsedVars)
data Inh_Stmts  = Inh_Stmts {labels_Inh_Stmts :: [Label],offset_Inh_Stmts :: Int,syms_Inh_Stmts :: Syms,uvars_Inh_Stmts :: UsedVars}
data Syn_Stmts  = Syn_Stmts {codes_Syn_Stmts :: CodeS,env_Syn_Stmts :: Env,labels_Syn_Stmts :: [Label],offset_Syn_Stmts :: Int,vars_Syn_Stmts :: UsedVars}
wrap_Stmts :: T_Stmts  ->
              Inh_Stmts  ->
              Syn_Stmts 
wrap_Stmts sem (Inh_Stmts _lhsIlabels _lhsIoffset _lhsIsyms _lhsIuvars )  =
    (let ( _lhsOcodes,_lhsOenv,_lhsOlabels,_lhsOoffset,_lhsOvars) =
             (sem _lhsIlabels _lhsIoffset _lhsIsyms _lhsIuvars )
     in  (Syn_Stmts _lhsOcodes _lhsOenv _lhsOlabels _lhsOoffset _lhsOvars ))
sem_Stmts_Cons :: T_Stmt  ->
                  T_Stmts  ->
                  T_Stmts 
sem_Stmts_Cons hd_ tl_  =
    (\ _lhsIlabels
       _lhsIoffset
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOenv :: Env
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _lhsOoffset :: Int
              _hdOlabels :: ([Label])
              _hdOoffset :: Int
              _hdOsyms :: Syms
              _hdOuvars :: UsedVars
              _tlOlabels :: ([Label])
              _tlOoffset :: Int
              _tlOsyms :: Syms
              _tlOuvars :: UsedVars
              _hdIcodes :: CodeS
              _hdIenv :: Env
              _hdIlabels :: ([Label])
              _hdIoffset :: Int
              _hdIvars :: UsedVars
              _tlIcodes :: CodeS
              _tlIenv :: Env
              _tlIlabels :: ([Label])
              _tlIoffset :: Int
              _tlIvars :: UsedVars
              _lhsOcodes =
                  {-# LINE 114 "./AG/CodeGeneration.ag" #-}
                  _hdIcodes . _tlIcodes
                  {-# LINE 1887 "Ag.hs" #-}
              _lhsOenv =
                  {-# LINE 66 "./AG/CodeGeneration.ag" #-}
                  _hdIenv ++ _tlIenv
                  {-# LINE 1891 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 116 "./AG/CodeGeneration.ag" #-}
                  _hdIvars `mergeUsedVars` _tlIvars
                  {-# LINE 1895 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _tlIlabels
                  {-# LINE 1899 "Ag.hs" #-}
              _lhsOoffset =
                  {-# LINE 36 "./AG/CodeGeneration.ag" #-}
                  _tlIoffset
                  {-# LINE 1903 "Ag.hs" #-}
              _hdOlabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 1907 "Ag.hs" #-}
              _hdOoffset =
                  {-# LINE 35 "./AG/CodeGeneration.ag" #-}
                  _lhsIoffset
                  {-# LINE 1911 "Ag.hs" #-}
              _hdOsyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 1915 "Ag.hs" #-}
              _hdOuvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 1919 "Ag.hs" #-}
              _tlOlabels =
                  {-# LINE 14 "./AG/CodeGeneration.ag" #-}
                  _hdIlabels
                  {-# LINE 1923 "Ag.hs" #-}
              _tlOoffset =
                  {-# LINE 35 "./AG/CodeGeneration.ag" #-}
                  _hdIoffset
                  {-# LINE 1927 "Ag.hs" #-}
              _tlOsyms =
                  {-# LINE 97 "./AG/CodeGeneration.ag" #-}
                  _lhsIsyms
                  {-# LINE 1931 "Ag.hs" #-}
              _tlOuvars =
                  {-# LINE 119 "./AG/CodeGeneration.ag" #-}
                  _lhsIuvars
                  {-# LINE 1935 "Ag.hs" #-}
              ( _hdIcodes,_hdIenv,_hdIlabels,_hdIoffset,_hdIvars) =
                  (hd_ _hdOlabels _hdOoffset _hdOsyms _hdOuvars )
              ( _tlIcodes,_tlIenv,_tlIlabels,_tlIoffset,_tlIvars) =
                  (tl_ _tlOlabels _tlOoffset _tlOsyms _tlOuvars )
          in  ( _lhsOcodes,_lhsOenv,_lhsOlabels,_lhsOoffset,_lhsOvars)))
sem_Stmts_Nil :: T_Stmts 
sem_Stmts_Nil  =
    (\ _lhsIlabels
       _lhsIoffset
       _lhsIsyms
       _lhsIuvars ->
         (let _lhsOcodes :: CodeS
              _lhsOenv :: Env
              _lhsOvars :: UsedVars
              _lhsOlabels :: ([Label])
              _lhsOoffset :: Int
              _lhsOcodes =
                  {-# LINE 114 "./AG/CodeGeneration.ag" #-}
                  id
                  {-# LINE 1955 "Ag.hs" #-}
              _lhsOenv =
                  {-# LINE 66 "./AG/CodeGeneration.ag" #-}
                  []
                  {-# LINE 1959 "Ag.hs" #-}
              _lhsOvars =
                  {-# LINE 116 "./AG/CodeGeneration.ag" #-}
                  []
                  {-# LINE 1963 "Ag.hs" #-}
              _lhsOlabels =
                  {-# LINE 15 "./AG/CodeGeneration.ag" #-}
                  _lhsIlabels
                  {-# LINE 1967 "Ag.hs" #-}
              _lhsOoffset =
                  {-# LINE 36 "./AG/CodeGeneration.ag" #-}
                  _lhsIoffset
                  {-# LINE 1971 "Ag.hs" #-}
          in  ( _lhsOcodes,_lhsOenv,_lhsOlabels,_lhsOoffset,_lhsOvars)))